import tqdm
import numpy as np
import click
import libertem_dectris


@click.command()
@click.argument('width', type=int, default=256)
@click.argument('height', type=int, default=256)
def main(width: int, height: int):
    conn = libertem_dectris.DectrisConnection(
        uri="tcp://localhost:9999",
        handle_path="/tmp/dectris_shm",
        frame_stack_size=32,
        num_slots=2000,
        bytes_per_frame=32*512,
        huge=False,
    )

    conn.start_passive()

    print("now arm and trigger the detector")
    while (res := conn.wait_for_arm(timeout=1.0)) is None:
        print("waiting...")

    config, series = res
    print(f"series {series}; config: {config}")

    cam_client = libertem_dectris.CamClient(conn.get_socket_path())

    tq = tqdm.tqdm(total=config.get_num_frames())

    try:
        while True:
            # get at most `max_size` frames as a stack
            # (might get less at the end of the acquisition)
            stack_handle = conn.get_next_stack(max_size=1024)

            # if the receiver is idle, stack_handle will be None here:
            if stack_handle is None:
                break

            tq.update(len(stack_handle))
            if False:
                cam_client.done(stack_handle)
                continue

            # the expected shape and data type:
            frame_shape = tuple(reversed(stack_handle.get_shape()))
            dtype = np.dtype(stack_handle.get_pixel_type()).newbyteorder(
                stack_handle.get_endianess()
            )

            # pre-allocate some memory for the pixel data:
            # (would be pulled out of the loop in real code)
            buf = np.zeros((len(stack_handle),) + frame_shape, dtype=dtype)

            # decompress into the pre-allocated buffer
            cam_client.decompress_frame_stack(stack_handle, out=buf)

            # free up the shared memory slot for this frame stack:
            cam_client.done(stack_handle)

            # we can still use the decompressed data:
            buf.sum()
    finally:
        conn.close()  # clean up background thread etc.
        cam_client.close()
    print("done")


if __name__ == "__main__":
    main()




import libertem_dectris
from libertem_live.detectors.dectris.DEigerClient import DEigerClient

if __name__ == "__main__":
    nimages = 256 * 256
    ec = DEigerClient('localhost', 8910)  # hostname and port of the DCU REST API
    ec.setDetectorConfig('ntrigger', 1)
    ec.setDetectorConfig('nimages', 1)
    ec.setDetectorConfig('trigger_mode', 'exte')
    ec.setDetectorConfig('ntrigger', nimages)
    result = ec.sendDetectorCommand('arm')
    sequence_id = result['sequence id'] 
    conn = libertem_dectris.DectrisConnection(
        uri="tcp://localhost:9999",
        handle_path="/tmp/dectris_shm",
        frame_stack_size=32,
        num_slots=2000,
        bytes_per_frame=512*512,
        huge=False,
    )

    conn.start(sequence_id)

    cam_client = libertem_dectris.CamClient(conn.get_socket_path())
    stack_handle = conn.get_next_stack(max_size=32)
    serialized = stack_handle.serialize()
    loaded = libertem_dectris.FrameStackHandle.deserialize(serialized)

    print(serialized)
    print(len(loaded))
    print(loaded.get_pixel_type())
    print(loaded.get_encoding())
    print(loaded.get_shape())

    cam_client.done(stack_handle)
    conn.close()


import time
import click
import perf_utils
import libertem_dectris
from libertem_live.detectors.dectris.DEigerClient import DEigerClient

@click.command()
@click.argument('nimages', type=int)
def main(nimages: int):
    ec = DEigerClient('localhost', 8910)

    ec.setDetectorConfig('ntrigger', 1)
    ec.setDetectorConfig('nimages', 1)
    ec.setDetectorConfig('trigger_mode', 'exte')
    ec.setDetectorConfig('ntrigger', nimages)

    result = ec.sendDetectorCommand('arm')
    sequence_id = result['sequence id']

    conn = libertem_dectris.DectrisConnection(
        uri="tcp://localhost:9999",
        handle_path="/tmp/dectris_shm",
        frame_stack_size=32,
        num_slots=2000,
        bytes_per_frame=512*512,
        huge=False,
    )

    conn.start(sequence_id)
    cam_client = libertem_dectris.CamClient(conn.get_socket_path())

    with perf_utils.perf('frame_iterator', output_dir='profiles') as perf_data:
        try:
            t0 = time.perf_counter()
            total = 0
            while True:
                stack_handle = conn.get_next_stack(max_size=512)
                if stack_handle is None:
                    break
                stack_handle.serialize()
                total += len(stack_handle)
                cam_client.done(stack_handle)
        finally:
            t1 = time.perf_counter()
            print(f"got {total} frames in {t1 - t0:.3f}s ({total/(t1-t0):.3f} fps); done, closing")
            conn.close()


if __name__ == "__main__":
    main()



import time
import click
import perf_utils
import libertem_dectris
from libertem_live.detectors.dectris.DEigerClient import DEigerClient
from libertem_live.detectors.dectris.acquisition import (
    DectrisAcquisition, DectrisDetectorConnection
)
from libertem_live.api import LiveContext
from libertem.udf.sumsigudf import SumSigUDF


@click.command()
@click.argument('width', type=int, default=256)
@click.argument('height', type=int, default=256)
def main(width: int, height: int):

    ctx = LiveContext()
    conn = DectrisDetectorConnection(
        api_host='localhost',
        api_port=8910,
        data_host='localhost',
        data_port=9999,
        frame_stack_size=32,
        num_slots=2000,
        bytes_per_frame=512*512,
        huge_pages=True,
    )

    print("now arm and trigger the detector")
    while (pending_aq := conn.wait_for_acquisition(timeout=10)) is None:
        print("waiting...")

    aq = DectrisAcquisition(
        conn=conn,
        nav_shape=(height, width),
        trigger=lambda x: None,
        frames_per_partition=1024,
        pending_aq=pending_aq,
    ) 
    aq = aq.initialize(ctx.executor)
    t0 = time.time()
    ctx.run_udf(
        dataset=aq,
        udf=SumSigUDF(),
    )
    t1 = time.time()
    print(t1-t0)
    print(f"{aq.shape.nav.size/(t1-t0)}")
    ctx.close()
    conn.close()
    print("done")


if __name__ == "__main__":
    main()




import time
import perf_utils
import libertem_dectris
from libertem_live.detectors.dectris.DEigerClient import DEigerClient
from libertem_live.detectors.dectris.acquisition import (
    DectrisAcquisition, DectrisDetectorConnection
)
from libertem_live.api import LiveContext
from libertem.udf.sumsigudf import SumSigUDF


if __name__ == "__main__":
    ctx = LiveContext()
    conn = DectrisDetectorConnection(
        api_host='localhost',
        api_port=8910,
        data_host='localhost',
        data_port=9999,
        frame_stack_size=32,
        num_slots=2000,
        bytes_per_frame=512*512,
        huge_pages=True,
    )

    aq = DectrisAcquisition(
        conn=conn,
        nav_shape=(256, 256),
        trigger=lambda x: None,
        frames_per_partition=1024,
        controller=conn.get_active_controller(
            trigger_mode='exte',
        )
    ) 
    aq = aq.initialize(ctx.executor)
    # warmup:
    ctx.run_udf(
        dataset=aq,
        udf=SumSigUDF(),
    )
    for i in range(10):
        t0 = time.time()
        ctx.run_udf(
            dataset=aq,
            udf=SumSigUDF(),
        )
        t1 = time.time()
        print(t1-t0)
        print(f"{aq.shape.nav.size/(t1-t0)}")
    ctx.close()
    conn.close()
    print("done")



import time
import libertem_dectris
from libertem_live.detectors.dectris.DEigerClient import DEigerClient

if __name__ == "__main__":
    # this is emulating a slow data consumer
    # if the number of slots is large enough, this should still work,
    # as the whole acquisition _can_ fit into shared memory
    conn = libertem_dectris.DectrisConnection(
        uri="tcp://localhost:9999",
        handle_path="/tmp/dectris_shm",
        frame_stack_size=32,
        num_slots=1000,
        bytes_per_frame=512*512,
        huge=False,
    )
    ec = DEigerClient('localhost', 8910)

    ec.setDetectorConfig('ntrigger', 1)
    ec.setDetectorConfig('nimages', 1)
    ec.setDetectorConfig('trigger_mode', 'exte')
    ec.setDetectorConfig('ntrigger', 256*256)

    result = ec.sendDetectorCommand('arm')
    sequence_id = result['sequence id']

    conn.start(series=sequence_id)
    cam_client = libertem_dectris.CamClient(conn.get_socket_path())

    while True:
        stack_handle = conn.get_next_stack(max_size=1024)
        if stack_handle is None:
            break
        print(f"{len(stack_handle)}")
        time.sleep(1.1)
        cam_client.done(stack_handle)

    conn.close()



import time
import click
import libertem_dectris
from libertem_live.detectors.dectris.DEigerClient import DEigerClient


@click.command()
@click.argument('nimages', type=int)
def main(nimages: int):
    ec = DEigerClient('localhost', 8910)

    ec.setDetectorConfig('ntrigger', 1)
    ec.setDetectorConfig('nimages', 1)
    ec.setDetectorConfig('trigger_mode', 'exte')
    ec.setDetectorConfig('ntrigger', nimages)

    result = ec.sendDetectorCommand('arm')
    sequence_id = result['sequence id']

    conn = libertem_dectris.DectrisConnection(
        uri="tcp://localhost:9999",
        handle_path="/tmp/dectris_shm",
        frame_stack_size=32,
        num_slots=2000,
        bytes_per_frame=512*512,
        huge=False,
    )

    conn.start(sequence_id)
    cam_client = libertem_dectris.CamClient(conn.get_socket_path())

    try:
        t0 = time.perf_counter()
        total = 0
        while True:
            stack_handle = conn.get_next_stack(max_size=32)
            if stack_handle is None:
                break
            stack_handle.serialize()
            total += len(stack_handle)
            cam_client.done(stack_handle)
    finally:
        t1 = time.perf_counter()
        print(f"got {total} frames in {t1 - t0:.3f}s ({total/(t1-t0):.3f} fps); done, closing")
        conn.close()


if __name__ == "__main__":
    main()
